SystemOrganization addCategory: #VersionChangesCalculator!

Object subclass: #ChangeCriteria
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

ChangeCriteria subclass: #ChangeByCyclomaticComplexity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByCyclomaticComplexity methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/21/2012 16:50'!
getNumber: aClass
	"Returns the cyclomatic complexity of the given class."

	^ aClass weightedMethodCount.
	! !

ChangeCriteria subclass: #ChangeByLinesNum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByLinesNum methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/15/2012 01:32'!
getNumber: aClass
	"Returns the number of lines of code of the given class."

	^ aClass numberOfLinesOfCode.
	! !

ChangeCriteria subclass: #ChangeByMethodsNum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeByMethodsNum methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/15/2012 01:32'!
getNumber: aClass
	"Returns the number of methods of the given class."

	^ aClass numberOfMethods.! !

!ChangeCriteria methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/15/2012 01:32'!
getNumber: aClass
	"comment stating purpose of message"

	^ 0.! !

Object subclass: #ChangeInfo
	instanceVariableNames: 'changeNum versionNum'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!ChangeInfo methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/22/2012 19:36'!
changeNum
	^changeNum.
! !

!ChangeInfo methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/22/2012 19:36'!
changeNum: change
	changeNum := change.
! !

!ChangeInfo methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/22/2012 19:37'!
versionNum
	^versionNum.
! !

!ChangeInfo methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/22/2012 19:37'!
versionNum: version
	versionNum := version.
! !

Object subclass: #VersionChangesCalculator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'VersionChangesCalculator'!

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/21/2012 16:44'!
calculateClassVersionChanges: classHistory withCriteria: changeCriteria numberOfVersions: numberOfVersions 
	"Calculates the change of a single class for each of its adjacent versions."
	
	| versionsChangesArray |
	versionsChangesArray := Array new: (numberOfVersions - 1).
	classHistory versions do: [:classVersion |
		| versionNum previousClassVersion diffValue |
		"Get the number of the version being processed 
		(the class may not exist in all the system versions)."
		versionNum := classVersion referenceVersion versionEntity name.
		"Get the previous class version to compare with."
		previousClassVersion := classVersion previousVersion.
		(previousClassVersion isNil)
			"If the previous version is not nil compare the change with the 
			given criteria."
			ifFalse: [
				| value1 value2 |
				value1 := changeCriteria getNumber: classVersion.
				value2 := changeCriteria getNumber: previousClassVersion.
				"Get the absolute value of the difference between the adjacent values"
				diffValue := (value1 - value2) abs.
			]
			ifTrue: [
				diffValue := changeCriteria getNumber: classVersion.
			].
		(versionNum asInteger > 1)
			"If the version being analyzed is not the first one."
			ifTrue: [
				"Add the change difference to the changes array."
				versionsChangesArray at: (versionNum - 1) put: diffValue.	
			].
	].	
	^versionsChangesArray.
	! !

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/21/2012 16:29'!
calculateClassesVersionChanges: hismoModel withCriteria: changeCriteria
	"Creates a dictionary with the name of all the classes in the Hismo Model as 
	keys and an array that contains information about each version."

	| classesChangesDict classesHistories versionsNum |
	classesChangesDict := Dictionary new.
	"Get the history for all the classes that are not inside a library (isStub not)."
	classesHistories := hismoModel allClassHistories select: [:each | each last isStub not].	
	versionsNum := hismoModel versions size.
	"Get the difference between adjacent classes versions, depending on the criteria."
	classesHistories do: [:classHistory |
		| className changesArray |
		className := classHistory name.
		changesArray := self calculateClassVersionChanges: classHistory withCriteria: changeCriteria numberOfVersions: versionsNum.
		classesChangesDict at: className put: changesArray.
	].
	^classesChangesDict.
 ! !

!VersionChangesCalculator methodsFor: 'as yet unclassified' stamp: 'FrancoSabadini 4/22/2012 19:58'!
createClassChangesLineChart: changesArray

	| chart |

	chart := ESDiagramRenderer new.
	(chart lineDiagram)
		y: #changeNum;
		yAxisLabel: 'Change Number';
		identifier: #versionNum;
		rotatedLabels: true;
		regularAxis;
		defaultColor: Color blue;
		"deviationValue: (changesArray collect: #changeNum) average;
		deviationDescription: 'average';"
		preferredAxisMinY: (changesArray minValue: #changeNum);
		models: changesArray.	
	^ chart open.! !
